$(function () {
  var that = this;

  var rowTemplate = '<tr class="custom-row">\n' +
    '  <td class="username">\n' +
    '    <input data-rule-required="true" data-msg-required="You must supply both a username and password." type="text" value=""/>\n' +
    '  </td>\n' +
    '  <td class="password">\n' +
    '    <input data-rule-required="true" data-msg-required="You must supply both a username and password." type="password" value=""/>\n' +
    '  </td>\n' +
    '  <td class="level">\n' +
    '    <select>\n' +
    '      <option value="READ" selected>Read</option>\n' +
    '      <option value="READWRITE">Read & write</option>\n' +
    '    </select>\n' +
    '  </td>\n' +
    '  <td class="note">\n' +
    '    <input type="text" placeholder="(notes)"/>\n' +
    '  </td>\n' +
    '  <td class="delete">\n' +
    '    <a href="#">Delete</a>\n' +
    '  </td>\n' +
    '</tr>\n';

  var fixCredNames = function () {
    var customRows = $('form#credentials table tbody tr.custom-row');
    var anonymousLevel = $('form#credentials #anonymousLevel').off('change').on('change', checkFormModified);
    if (customRows.length === 0) {
      anonymousLevel.val('READWRITE').attr('disabled', 'yes');
    } else {
      anonymousLevel.removeAttr('disabled');
      customRows.each(function (idx) {
        $(this).find('.username input').attr('name', 'username[' + idx + ']').off('keyup').on('keyup', checkFormModified);
        $(this).find('.password input').attr('name', 'password[' + idx + ']').off('keyup').on('keyup', checkFormModified);
        $(this).find('.level select').attr('name', 'level[' + idx + ']').off('change').on('change', checkFormModified);
        $(this).find('.delete a').off('click').on('click', function () {
          var button = $(this);
          Modal.open('Are you sure you want to remove access for this user?\n\n', ['Yes', 'No'], function (result) {
            if (result === 'Yes') {
              button.parents('tr').remove();
              fixCredNames();
              checkFormModified();
            }
          });
          return false;
        });
        $(this).find('.note input').attr('name', 'note[' + idx + ']').off('keyup').on('keyup', checkFormModified);
      });
    }
    validate();
  };

  var disabledAvoidance = function(element, fn) {
    var isReadonly = hasAttribute(element, 'disabled');
    if (isReadonly === true) {
      element.removeAttr('disabled');
    }
    fn();
    if (isReadonly === true) {
      element.attr('disabled', 'yes');
    }
  };

  var checkFormModified = function () {
    var anonymousLevel = $('form#credentials #anonymousLevel');
    disabledAvoidance(anonymousLevel, function() {
      if (that.state.state !== that.formState().state) {
        that.resetButton.removeAttr('disabled');
      } else {
        that.resetButton.attr('disabled', 'yes');
      }
    });
  };

  // Add click
  $('#add-cred').on('click', function () {
    $('form#credentials table tbody').append(rowTemplate);
    fixCredNames();
    checkFormModified();
    $('form#credentials table tbody tr.custom-row .username:last input').focus();
    return false;
  });

  that.resetButton = $('form#credentials a#reset');

  that.resetButton.on('click', function () {
    if (!hasAttribute($(this), 'disabled')) {
      $('form#credentials table tbody').html(that.state.html);
      fixCredNames();
      // Apply values from state
      that.state.attr.map(function (entry) {
        $('form#credentials *[name="' + entry.name + '"]').val(entry.value);
      });
      checkFormModified();
    }
    return false;
  });

  var hasAttribute = function (elem, attr) {
    var a = $(elem).attr(attr);
    return typeof a !== typeof undefined && a !== false;
  };

  that.form = $('form#credentials');

  var uniqueUsernames = function(form) {
    var result = form.find('tr.custom-row').map(function() {
      return $(this).find("input[name^='username']").val();
    }).toArray();
    var unique = result.every(function(v,i) {
      return result.indexOf(v) === i;
    });
    if (!unique) {
      errorMsg('Usernames must be unique.', form.find('.msg'));
    }
    return unique;
  };

  // Validation
  var validate = function () {
    that.form.validate({
      ignore: '',
      onfocusout: false,
      onkeyup: false,
      errorPlacement: function (error, element) {
      },
      showErrors: function (errorMap, errorList) {
        var arrayLength = errorList.length;
        for (var i = 0; i < arrayLength; i++) {
          if (i === 0) {
            errorMsg(errorList[i].message, that.form.find('div.msg'));
          }
          $(errorList[i].element).addClass('error');
        }
        this.defaultShowErrors();
      },
      highlight: function (element, errorClass) {
        $(element).addClass(errorClass);
      },
      unhighlight: function (element, errorClass) {
        $(element).removeClass(errorClass);
      },
      submitHandler: function (form, event) {
        event.preventDefault();
        var $form = $(form)

        if (uniqueUsernames($form)) {
          // We have to hack about with the possibly disabled select, so it still gets posted
          var anonymousLevel = $('form#credentials #anonymousLevel');
          disabledAvoidance(anonymousLevel, function() {
            var payload = $form.serialize();
            jQuery.post($form.attr("action"), payload)
              .done(function () {
                successMsg($form.data('successMsg'), $form.find('.msg'));
                that.state = that.formState();
                fixCredNames();
                checkFormModified();
              })
              .fail(function() {
                errorMsg($form.data('failureMsg'), $form.find('.msg'));
              });
          });

        }

        // Re-capture state
        return false;
      }
    });
  };

  that.formState = function () {
    return {
      state: $('form#credentials').serialize(),
      html: $('form#credentials table tbody').html(),
      attr: $('form#credentials').serializeArray()
    };
  };

  that.state = that.formState();
  fixCredNames();
  checkFormModified();
});